package net.h34t.filemure

import net.h34t.filemure.classification.AIClassifier
import org.apache.pdfbox.Loader
import org.apache.pdfbox.rendering.PDFRenderer
import org.apache.pdfbox.text.PDFTextStripper
import java.io.ByteArrayOutputStream
import javax.imageio.ImageIO

class ContentExtractor(
    val aiClassifier: AIClassifier
) {

    fun extractImage(contentType: String, imageBytes: ByteArray): String {
        val text = aiClassifier.classifyImage(
            prompt = "Analyze the given image and concise summary of its contents, including the type of document depicted (invoice, certificate, doctor's note, ...).",
            contentType = contentType,
            data = imageBytes,
        )

        println("AI image classification:\n\"\"\"\n$text\n\"\"\"")

        return text
    }

    fun extractPdf(pdfBytes: ByteArray): String {
        val doc = Loader.loadPDF(pdfBytes)
        val text = PDFTextStripper().getText(doc)

        return if (text.isNotBlank()) {
            text
        } else {
            val renderer = PDFRenderer(doc)
            val bi = renderer.renderImage(0)
            val baos = ByteArrayOutputStream()

            ImageIO.write(bi, "JPG", baos)

            // for debugging
            // ImageIO.write(bi, "JPG", File.createTempFile("pdfimage-", ".jpg", File(".")))

            extractImage("image/jpeg", baos.toByteArray())
        }
    }

    fun extractPlain(bytes: ByteArray): String {
        return bytes.toString(Charsets.UTF_8)
    }
}