package net.h34t.filemure

import java.time.LocalDate
import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import java.time.format.DateTimeParseException

object DateGuesser {

    private val dateTimePatterns = listOf(
        Regex("\\d{4}-\\d{2}-\\d{2}_\\d{2}:\\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd'_'HH:mm:ss"),
        Regex("\\d{4}-\\d{2}-\\d{2}_\\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd'_'HH:mm"),

        Regex("\\d{4}-\\d{2}-\\d{2}-\\d{2}:\\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd'-'HH:mm:ss"),
        Regex("\\d{4}-\\d{2}-\\d{2}-\\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd'-'HH:mm"),

        Regex("\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd' 'HH:mm:ss"),
        Regex("\\d{4}-\\d{2}-\\d{2} \\d{2}:\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd' 'HH:mm"),

        // ---

        Regex("\\d{4}\\d{2}\\d{2}_\\d{2}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd'_'HHmmss"),
        Regex("\\d{4}\\d{2}\\d{2}_\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd'_'HHmm"),

        Regex("\\d{4}\\d{2}\\d{2}-\\d{2}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd'-'HHmmss"),
        Regex("\\d{4}\\d{2}\\d{2}-\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd'-'HHmm"),

        Regex("\\d{4}\\d{2}\\d{2} \\d{2}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd' 'HHmmss"),
        Regex("\\d{4}\\d{2}\\d{2} \\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd' 'HHmm"),

        Regex("\\d{4}\\d{2}\\d{2}\\d{2}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMddHHmmss"),
        Regex("\\d{4}\\d{2}\\d{2}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMddHHmm"),
    )

    private val datePatterns = listOf(
        Regex("\\d{4}-\\d{2}-\\d{2}") to DateTimeFormatter.ofPattern("yyyy-MM-dd"),
        Regex("\\d{4}_\\d{2}_\\d{2}") to DateTimeFormatter.ofPattern("yyyy_MM_dd"),
        Regex("\\d{4}\\d{2}\\d{2}") to DateTimeFormatter.ofPattern("yyyyMMdd"),
        Regex("\\d{2}\\.\\d{2}\\.\\d{4}") to DateTimeFormatter.ofPattern("dd.MM.YYYY"),
        Regex("\\d{2}\\.\\d{2}\\.\\d{2}") to DateTimeFormatter.ofPattern("dd.MM.yy"),

        )

    private fun guessDateTime(filename: String) = dateTimePatterns.asSequence().mapNotNull { rxPattern ->
        rxPattern.first.findAll(filename).mapNotNull { mr ->
            try {
                LocalDateTime.parse(mr.groups[0]?.value!!, rxPattern.second)
            } catch (_: DateTimeParseException) {
                null
            }
        }.firstOrNull()
    }.firstOrNull()

    private fun guessDate(filename: String) = datePatterns.asSequence().mapNotNull { rxPattern ->
        rxPattern.first.findAll(filename).mapNotNull { mr ->
            try {
                LocalDate.parse(mr.groups[0]?.value!!, rxPattern.second)
            } catch (_: DateTimeParseException) {
                null
            }
        }.firstOrNull()
    }.firstOrNull()

    fun guess(filename: String): LocalDateTime? {
        return guessDateTime(filename) ?: (guessDate(filename)?.atTime(0, 0))
    }
}