package net.h34t.filemure.controller

import io.javalin.http.BadRequestResponse
import io.javalin.http.Context
import net.h34t.filemure.*
import net.h34t.filemure.repository.SqliteRepository
import net.h34t.filemure.tpl.Frame
import net.h34t.filemure.tpl.Login
import net.h34t.filemure.tpl.Register

class AccountController(private val modifiers: TemplateModifiers, private val repository: SqliteRepository) {

    fun formLogin(ctx: Context) {
        val loginError = ctx.queryParam("login_error") != null

        ctx.tempolin(
            Frame(
                modifiers = modifiers,
                title = "Login",
                target = "",
                back = "",
                logout = false,
                content = Login(
                    loginError = loginError
                )
            )
        )
    }

    fun doLogin(ctx: Context) {
        val username = ctx.formParam("username") ?: throw BadRequestResponse()
        val password = ctx.formParam("password") ?: throw BadRequestResponse()

        val acc = repository.loginAccount(email = username, password = password.trim().toCharArray())

        if (acc != null) {
            ctx.setSession(Session(id = acc.id, email = acc.email))
            ctx.redirectPRG("/")
        } else {
            ctx.setSession(null)
            ctx.redirectPRG("/login/?login_error")
        }
    }

    fun doLogout(ctx: Context) {
        ctx.setSession(null)
        ctx.redirectPRG("/login")
    }

    fun formRegister(ctx: Context) {
        val registerError = ctx.queryParam("register_error") != null

        ctx.tempolin(
            Frame(
                modifiers = modifiers,
                title = "Register",
                target = "",
                back = "/",
                logout = false,
                content = Register(
                    registerError = registerError
                )
            )
        )
    }

    fun doRegister(ctx: Context) {
        val username = ctx.formParam("username") ?: throw BadRequestResponse()
        val password = ctx.formParam("password") ?: throw BadRequestResponse()

        try {
            require(username.isNotBlank())
            require(password.isNotBlank() && password.length >= 6)

            repository.addAccount(email = username, password = password.trim().toCharArray())
            ctx.redirectPRG("/login")
        } catch (e: Exception) {
            e.printStackTrace()
            ctx.redirectPRG("/register?register_error")
        }
    }
}