package net.h34t.filemure.controller

import io.javalin.http.Context
import net.h34t.filemure.*
import net.h34t.filemure.repository.SqliteRepository
import net.h34t.filemure.tpl.Frame
import net.h34t.filemure.tpl.Search
import net.h34t.filemure.tpl.Tags

class SearchController(val modifiers: TemplateModifiers, val repository: SqliteRepository) {

    fun search(ctx: Context) {
        val session = ctx.requireSession()

        val q = ctx.queryParam("q")

        val documents = repository.searchDocuments(accountId = session.id, query = "%$q%")

        ctx.tempolin(
            Frame(
                modifiers = modifiers,
                title = "Search",
                target = "document",
                back = "/",
                logout = true,
                content = Search(
                    modifiers = modifiers,
                    search = q ?: "",
                    document = {
                        documents.map { d ->
                            DocumentBlock(
                                extId = d.extId.value,
                                title = d.title,
                                referenceDate = d.referenceDate.formatHumanShort()
                            )
                        }.asSequence()
                    }
                )
            )
        )
    }

    fun tags(ctx: Context) {
        val session = ctx.requireSession()

        val t = ctx.queryParams("t").map { Tag.of(it) }

        val tags = repository.getAllTags(session.id)

        val documents = repository.getDocumentsByTags(accountId = session.id, tags = t)

        ctx.tempolin(
            Frame(
                modifiers = modifiers,
                title = "Search",
                target = "document",
                back = "/",
                logout = true,
                content = Tags(
                    modifiers = modifiers,
                    tag = { tags.map { TagBlock(tag = it.value) }.asSequence() },
                    document = {
                        documents.map { d ->
                            DocumentBlock(
                                extId = d.extId.value,
                                title = d.title,
                                referenceDate = d.referenceDate.formatHumanShort()
                            )
                        }.asSequence()
                    }
                )
            )
        )
    }
}