package net.h34t.filemure.controller

import io.javalin.http.Context
import net.h34t.filemure.ContentExtractor
import net.h34t.filemure.TemplateModifiers
import net.h34t.filemure.repository.SqliteRepository
import net.h34t.filemure.requireSession

class UploadController(
    private val modifiers: TemplateModifiers,
    private val repository: SqliteRepository,
    private val contentExtractor: ContentExtractor
) {

    fun upload(ctx: Context) {
        val session = ctx.requireSession()

        val accountid = session.id

        val target = ctx.queryParam("target")

        val files = ctx.uploadedFiles()

        val extIds = files.map {
            it.contentAndClose { contentStream ->
                val content = contentStream.readAllBytes()
                val contentType = it.contentType()

                val contentExtracted = when (contentType) {
                    "application/pdf" -> contentExtractor.extractPdf(content)
                    "text/plain" -> contentExtractor.extractPlain(content)
                    else -> ""
                }

                repository.addFileToLimbo(
                    accountid,
                    it.filename(),
                    contentType,
                    it.size(),
                    contentExtracted,
                    content
                ).extId
            }
        }

        when (target) {
            "document" -> {
                ctx.status(200)
                ctx.json(
                    Result(
                        files = files.size,
                        redirect = "/document/new?${extIds.joinToString("&") { "file_id=$it" }}"
                    )
                )
            }

            "limbo" -> {
                ctx.status(200)
                ctx.json(
                    Result(
                        files = files.size,
                        redirect = "/limbo"
                    )
                )
            }
        }
    }

    data class Result(
        val files: Int,
        val redirect: String
    )
}
