package net.h34t.filemure

import java.time.LocalDateTime
import java.time.format.DateTimeFormatter
import kotlin.test.Test
import kotlin.test.assertEquals

class DateGuesserTest {

    private val dtf = DateTimeFormatter.ISO_DATE_TIME

    private fun toDateTime(value: String?): LocalDateTime? = value?.let { LocalDateTime.parse(value, dtf) }

    @Test
    fun test_date() {
        val values = listOf(
            "2023-10-05T00:00" to "20231005_Form.pdf",
            "2023-10-05T00:00" to "File-20231005_Form.pdf",
            "2023-10-05T00:00" to "File-2023-10-05_Form.pdf",
            "2023-10-05T00:00" to "File-2023_10_05_Form.pdf",
            "2023-10-05T00:00" to "File-2023_10_05.pdf",
            "2023-10-05T10:15" to "File-20231005-101500_Form.pdf",
            "2023-10-05T11:22:33" to "File.20231005-112233.pdf",
            "2022-11-11T00:00:00" to "File.20221111000000.pdf",
            "8888-11-11T00:00:00" to "File.88881111000000.pdf",
            null to "Document.pdf",
        )

        values.forEach { value ->
            assertEquals(
                toDateTime(value.first),
                DateGuesser.guess(value.second),
                "Guessing the date failed for ${value.first} -> ${value.second}"
            )
        }
    }
}